@@ -81,6 +81,8 @@ my $XML_TOKEN_RE = qr/
     )??
 /xis;
 
+sub after { shift->_add(1, @_) }
+
 sub all_text {
     my $self = shift;
 
@@ -124,6 +126,8 @@ sub attrs {
     return $tree->[2];
 }
 
+sub before { shift->_add(0, @_) }
+
 sub children {
     my $self = shift;
 
@@ -366,6 +370,41 @@ sub type {
     return $self;
 }
 
+sub _add {
+    my ($self, $offset, $new) = @_;
+
+    # Parse
+    $new = ref $new ? $new->tree : $self->_parse_xml($new);
+
+    # Tree
+    my $tree = $self->tree;
+
+    # Root
+    return $self if $tree->[0] eq 'root';
+
+    # Parent
+    my $parent = $tree->[3];
+
+    # Siblings
+    my @new;
+    for my $e (@$new[1 .. $#$new]) {
+        $e->[3] = $parent if $e->[0] eq 'tag';
+        push @new, $e;
+    }
+
+    # Find
+    my $i = $parent->[0] eq 'root' ? 1 : 4;
+    for my $e (@$parent[$i .. $#$parent]) {
+        last if $e == $tree;
+        $i++;
+    }
+
+    # Add
+    splice @$parent, $i + $offset, 0, @new;
+
+    return $self;
+}
+
 # Woah! God is so in your face!
 # Yeah, he's my favorite fictional character.
 sub _cdata {
@@ -1310,6 +1349,14 @@ Document Object Model.
 L<Mojo::DOM> inherits all methods from L<Mojo::Base> and implements the
 following new ones.
 
+=head2 C<after>
+
+    $dom = $dom->after('<p>Hi!</p>');
+
+Add after element.
+
+    $dom->parse('<div><h1>A</h1></div>')->at('h1')->after('<h2>B</h2>');
+
 =head2 C<all_text>
 
     my $text = $dom->all_text;
@@ -1328,6 +1375,14 @@ Find a single element with CSS3 selectors.
 
 Element attributes.
 
+=head2 C<before>
+
+    $dom = $dom->before('<p>Hi!</p>');
+
+Add before element.
+
+    $dom->parse('<div><h2>A</h2></div>')->at('h2')->before('<h1>B</h1>');
+
 =head2 C<children>
 
     my $children = $dom->children;
@@ -1375,12 +1430,16 @@ Parse XML document.
 
 Replace elements.
 
+    $dom->parse('<div><h1>A</h1></div>')->at('h1')->replace('<h2>B</h2>');
+
 =head2 C<replace_inner>
 
     $dom = $dom->replace_inner('test');
 
 Replace element content.
 
+    $dom->parse('<div><h1>A</h1></div>')->at('h1')->replace_inner('B');
+
 =head2 C<root>
 
     my $root = $dom->root;
