@@ -12,11 +12,13 @@ use IO::File;
 use IO::Poll qw/POLLERR POLLHUP POLLIN POLLOUT/;
 use IO::Socket;
 use Mojo::ByteStream 'b';
+use Mojo::URL;
 use Scalar::Util 'weaken';
 use Socket qw/IPPROTO_TCP TCP_NODELAY/;
 use Time::HiRes 'time';
 
-use constant CHUNK_SIZE => $ENV{MOJO_CHUNK_SIZE} || 262144;
+# Debug
+use constant DEBUG => $ENV{MOJO_IOLOOP_DEBUG} || 0;
 
 # Epoll support requires IO::Epoll
 use constant EPOLL => ($ENV{MOJO_POLL} || $ENV{MOJO_KQUEUE})
@@ -98,8 +100,38 @@ AnqxHi90n/p912ynLg2SjBq+03GaECeGzC/QqKK2gtA=
 -----END RSA PRIVATE KEY-----
 EOF
 
-__PACKAGE__->attr([qw/accept_timeout connect_timeout/] => 5);
-__PACKAGE__->attr(max_connections                      => 1000);
+# DNS server (default to Google Public DNS)
+our $DNS_SERVER = '8.8.8.8';
+
+# Try to detect DNS server
+if (-r '/etc/resolv.conf') {
+    my $file = IO::File->new;
+    $file->open('< /etc/resolv.conf');
+    for my $line (<$file>) {
+        if ($line =~ /^nameserver\s+(\S+)$/) {
+
+            # New DNS server
+            $DNS_SERVER = $1;
+
+            # Debug
+            warn qq/DETECTED DNS SERVER ($DNS_SERVER)\n/ if DEBUG;
+        }
+    }
+}
+
+# DNS record types
+my $DNS_TYPES = {
+    A    => 0x0001,
+    AAAA => 0x001c,
+    TXT  => 0x0010
+};
+
+# "localhost"
+our $LOCALHOST = '127.0.0.1';
+
+__PACKAGE__->attr([qw/accept_timeout connect_timeout dns_timeout/] => 3);
+__PACKAGE__->attr(dns_server => sub { $ENV{MOJO_DNS_SERVER} || $DNS_SERVER });
+__PACKAGE__->attr(max_connections => 1000);
 __PACKAGE__->attr([qw/on_idle on_tick/]);
 __PACKAGE__->attr(
     [qw/on_lock on_unlock/] => sub {
@@ -153,44 +185,19 @@ sub connect {
     # TLS check
     return if $args->{tls} && !TLS;
 
-    # Options
-    my %options = (
-        PeerAddr => $args->{address},
-        PeerPort => $args->{port} || ($args->{tls} ? 443 : 80),
-        Proto    => 'tcp',
-        Type     => SOCK_STREAM,
-        %{$args->{args} || {}}
-    );
+    # Protocol
+    $args->{proto} ||= 'tcp';
 
-    # New connection
-    my $class = IPV6 ? 'IO::Socket::IP' : 'IO::Socket::INET';
-    return unless my $socket = $args->{socket} || $class->new(%options);
-    my $id = "$socket";
-
-    # File descriptor
-    return unless defined(my $fd = fileno $socket);
-    $self->{_fds}->{$fd} = $id;
-
-    # Add connection
-    my $c = $self->{_cs}->{$id} = {
+    # Connection
+    my $c = {
         buffer     => b(),
         on_connect => $args->{on_connect}
           || $args->{connect_cb}
           || $args->{cb},
-        connecting => 1,
-        socket     => $socket
+        connecting => 1
     };
-
-    # Non blocking
-    $socket->blocking(0);
-
-    # Disable Nagle's algorithm
-    setsockopt $socket, IPPROTO_TCP, TCP_NODELAY, 1;
-
-    # Timeout
-    $c->{connect_timer} =
-      $self->timer($self->connect_timeout =>
-          sub { shift->_error($id, 'Connect timeout.') });
+    (my $id) = "$c" =~ /0x([\da-f]+)/;
+    $self->{_cs}->{$id} = $c;
 
     # Register callbacks
     for my $name (qw/error hup read/) {
@@ -199,11 +206,19 @@ sub connect {
         $self->$event($id => $cb) if $cb;
     }
 
-    # Add socket to poll
-    $self->_not_writing($id);
+    # Lookup
+    if (my $address = $args->{address}) {
+        $self->lookup(
+            $address => sub {
+                my $self = shift;
+                $args->{address} = shift || $args->{address};
+                $self->_connect($id, $args);
+            }
+        );
+    }
 
-    # Start TLS
-    if ($args->{tls}) { return unless $id = $self->start_tls($id => $args) }
+    # Connect
+    else { $self->_connect($id, $args) }
 
     return $id;
 }
@@ -285,6 +300,17 @@ sub listen {
     my $fd;
     if ($ENV{MOJO_REUSE} =~ /(?:^|\,)$reuse\:(\d+)/) { $fd = $1 }
 
+    # Connection
+    my $c = {
+        file => $args->{file} ? 1 : 0,
+        on_accept => $args->{on_accept} || $args->{accept_cb} || $args->{cb},
+        on_error  => $args->{on_error}  || $args->{error_cb},
+        on_hup    => $args->{on_hup}    || $args->{hup_cb},
+        on_read   => $args->{on_read}   || $args->{read_cb},
+    };
+    (my $id) = "$c" =~ /0x([\da-f]+)/;
+    $self->{_listen}->{$id} = $c;
+
     # Listen on UNIX domain socket
     my $socket;
     if (defined $file) {
@@ -322,18 +348,11 @@ sub listen {
         $reuse = ",$reuse" if length $ENV{MOJO_REUSE};
         $ENV{MOJO_REUSE} .= "$reuse:$fd";
     }
-    my $id = "$socket";
     $self->{_fds}->{$fd} = $id;
 
-    # Add listen socket
-    my $c = $self->{_listen}->{$id} = {
-        file => $args->{file} ? 1 : 0,
-        on_accept => $args->{on_accept} || $args->{accept_cb} || $args->{cb},
-        on_error  => $args->{on_error}  || $args->{error_cb},
-        on_hup    => $args->{on_hup}    || $args->{hup_cb},
-        on_read   => $args->{on_read}   || $args->{read_cb},
-        socket    => $socket
-    };
+    # Socket
+    $c->{socket} = $socket;
+    $self->{_reverse}->{$socket} = $id;
 
     # TLS options
     $c->{tls} = {
@@ -362,6 +381,38 @@ sub local_info {
     return {address => $socket->sockhost, port => $socket->sockport};
 }
 
+sub lookup {
+    my ($self, $name, $cb) = @_;
+
+    # "localhost"
+    return $self->$cb($LOCALHOST) if $name eq 'localhost';
+
+    # IPv4
+    $self->resolve(
+        $name, 'A',
+        sub {
+            my ($self, $results) = @_;
+
+            # Success
+            return $self->$cb($results->[0]) if $results->[0];
+
+            # IPv6
+            $self->resolve(
+                $name, 'AAAA',
+                sub {
+                    my ($self, $results) = @_;
+
+                    # Success
+                    return $self->$cb($results->[0]) if $results->[0];
+
+                    # Pass through
+                    $self->$cb();
+                }
+            );
+        }
+    );
+}
+
 sub on_error { shift->_add_event('error', @_) }
 sub on_hup   { shift->_add_event('hup',   @_) }
 sub on_read  { shift->_add_event('read',  @_) }
@@ -381,6 +432,9 @@ sub one_tick {
     # Loop
     my $loop = $self->_prepare_loop;
 
+    # Reverse map
+    my $r = $self->{_reverse};
+
     # Events
     my (@error, @hup, @read, @write);
 
@@ -419,16 +473,16 @@ sub one_tick {
         $loop->poll($timeout);
 
         # Read
-        push @read, "$_" for $loop->handles(EPOLL_POLLIN);
+        push @read, $r->{$_} for $loop->handles(EPOLL_POLLIN);
 
         # Write
-        push @write, "$_" for $loop->handles(EPOLL_POLLOUT);
+        push @write, $r->{$_} for $loop->handles(EPOLL_POLLOUT);
 
         # Error
-        push @error, "$_" for $loop->handles(EPOLL_POLLERR);
+        push @error, $r->{$_} for $loop->handles(EPOLL_POLLERR);
 
         # HUP
-        push @hup, "$_" for $loop->handles(EPOLL_POLLHUP);
+        push @hup, $r->{$_} for $loop->handles(EPOLL_POLLHUP);
     }
 
     # Poll
@@ -436,16 +490,16 @@ sub one_tick {
         $loop->poll($timeout);
 
         # Read
-        push @read, "$_" for $loop->handles(POLLIN);
+        push @read, $r->{$_} for $loop->handles(POLLIN);
 
         # Write
-        push @write, "$_" for $loop->handles(POLLOUT);
+        push @write, $r->{$_} for $loop->handles(POLLOUT);
 
         # Error
-        push @error, "$_" for $loop->handles(POLLERR);
+        push @error, $r->{$_} for $loop->handles(POLLERR);
 
         # HUP
-        push @hup, "$_" for $loop->handles(POLLHUP);
+        push @hup, $r->{$_} for $loop->handles(POLLHUP);
     }
 
     # Read
@@ -491,6 +545,140 @@ sub remote_info {
     return {address => $socket->peerhost, port => $socket->peerport};
 }
 
+sub resolve {
+    my ($self, $name, $type, $cb) = @_;
+
+    # Regex
+    my $ipv4 = $Mojo::URL::IPV4_RE;
+    my $ipv6 = $Mojo::URL::IPV6_RE;
+
+    # Type
+    my $t = $DNS_TYPES->{$type};
+
+    # Server
+    my $server = $self->dns_server;
+
+    # No lookup required or record type not supported
+    unless ($server && $t && $name !~ $ipv4 && $name !~ $ipv6) {
+        $self->$cb([]);
+        return $self;
+    }
+
+    # Debug
+    warn "RESOLVE $type $name ($server)\n" if DEBUG;
+
+    # Timer
+    my $timer;
+
+    # Transaction
+    my $tx = int rand 0x10000;
+
+    # Request
+    my $id = $self->connect(
+        address    => $server,
+        port       => 53,
+        proto      => 'udp',
+        on_connect => sub {
+            my ($self, $id) = @_;
+
+            # Header (one question with recursion)
+            my $req = pack 'nnnnnn', $tx, 0x0100, 1, 0, 0, 0;
+
+            # Query (Internet)
+            for my $part (split /\./, $name) {
+                $req .= pack 'C/a', $part if defined $part;
+            }
+            $req .= pack 'Cnn', 0, $t, 0x0001;
+
+            # Write
+            $self->write($id => $req);
+        },
+        on_error => sub {
+            my ($self, $id) = @_;
+
+            # Debug
+            warn "FAILED $type $name ($server)\n" if DEBUG;
+
+            $self->drop($timer) if $timer;
+            $self->$cb([]);
+        },
+        on_read => sub {
+            my ($self, $id, $chunk) = @_;
+
+            # Cleanup
+            $self->drop($id);
+            $self->drop($timer) if $timer;
+
+            # Packet
+            my @packet = unpack 'nnnnnnA*', $chunk;
+
+            # Wrong response
+            return $self->$cb([]) unless $packet[0] eq $tx;
+
+            # Content
+            my $content = $packet[6];
+
+            # Questions
+            for (1 .. $packet[2]) {
+                my $n;
+                do { ($n, $content) = unpack 'C/aA*', $content } while ($n);
+                $content = (unpack 'nnA*', $content)[2];
+            }
+
+            # Answers
+            my @answers;
+            for (1 .. $packet[3]) {
+                my ($t, $a, $answer);
+                ($t, $a, $content) = (unpack 'nnnNn/AA*', $content)[1, 4, 5];
+
+                # A
+                if ($t eq $DNS_TYPES->{A}) {
+                    $answer = join('.', unpack 'C*', $a);
+                }
+
+                # AAAA
+                elsif ($t eq $DNS_TYPES->{AAAA}) {
+                    $answer = sprintf '%x:%x:%x:%x:%x:%x:%x:%x',
+                      unpack('n*', $a);
+                }
+
+                # TXT
+                elsif ($t eq $DNS_TYPES->{TXT}) {
+                    $answer = unpack '(C/a*)*', $a;
+                }
+
+                next unless defined $answer;
+                push @answers, $answer;
+
+                # Debug
+                warn "ANSWER $answer\n" if DEBUG;
+            }
+
+            # Done
+            $self->$cb(\@answers);
+        }
+    );
+
+    # Timer
+    $timer = $self->timer(
+        $self->dns_timeout => sub {
+            my $self = shift;
+
+            # Debug
+            warn "RESOLVE TIMEOUT ($server)\n" if DEBUG;
+
+            # Disable
+            $self->dns_server(undef);
+
+            # Abort
+            $self->drop($id);
+            $self->$cb([]);
+        }
+    );
+
+    return $self;
+}
+
 sub singleton { $LOOP ||= shift->new(@_) }
 
 sub start {
@@ -533,6 +721,7 @@ sub start_tls {
     my $fd = fileno $socket;
 
     # Cleanup
+    delete $self->{_reverse}->{$socket};
     my $writing = delete $c->{writing};
     my $loop    = $self->_prepare_loop;
     if (KQUEUE) {
@@ -545,18 +734,13 @@ sub start_tls {
     $self->drop($id) and return
       unless my $new = IO::Socket::SSL->start_SSL($socket, %options);
 
-    # Update file descriptor
-    delete $self->{_fds}->{$fd};
-    $fd = fileno $new;
-    $self->{_fds}->{$fd} = "$new";
-
     # Upgrade
-    $c->{socket}         = $new;
-    $self->{_cs}->{$new} = delete $self->{_cs}->{$id};
-    $c->{tls_connect}    = 1;
-    $self->_writing("$new");
+    $c->{socket}              = $new;
+    $self->{_reverse}->{$new} = $id;
+    $c->{tls_connect}         = 1;
+    $self->_writing($id);
 
-    return "$new";
+    return $id;
 }
 
 sub stop { delete shift->{_running} }
@@ -587,7 +771,7 @@ sub timer {
     my $timer = {after => $after, cb => $cb, started => time};
 
     # Add timer
-    my $id = "$timer";
+    (my $id) = "$timer" =~ /0x([\da-f]+)/;
     $self->{_ts}->{$id} = $timer;
 
     return $id;
@@ -629,27 +813,32 @@ sub _accept {
     # Unlock
     $self->on_unlock->($self);
 
+    # Reverse map
+    my $r = $self->{_reverse};
+
     # Listen
-    my $l = $self->{_listen}->{$listen};
+    my $l = $self->{_listen}->{$r->{$listen}};
 
     # Weaken
     weaken $self;
 
+    # Connection
+    my $c = {
+        accepting => 1,
+        buffer    => b(),
+    };
+    (my $id) = "$c" =~ /0x([\da-f]+)/;
+    $self->{_cs}->{$id} = $c;
+
     # TLS handshake
     my $tls = $l->{tls};
     if ($tls) {
         $tls->{SSL_error_trap} = sub { $self->_drop_immediately(shift) };
         $socket = IO::Socket::SSL->start_SSL($socket, %$tls);
     }
-
-    # Add connection
-    my $id = "$socket";
-    my $c = $self->{_cs}->{$id} = {
-        accepting => 1,
-        buffer    => b(),
-        socket    => $socket
-    };
     $c->{tls_accept} = 1 if $tls;
+    $c->{socket}     = $socket;
+    $r->{$socket}    = $id;
 
     # Timeout
     $c->{accept_timer} =
@@ -704,6 +893,49 @@ sub _add_event {
     return $self;
 }
 
+sub _connect {
+    my ($self, $id, $args) = @_;
+
+    # Connection
+    return unless my $c = $self->{_cs}->{$id};
+
+    # Options
+    my %options = (
+        PeerAddr => $args->{address},
+        PeerPort => $args->{port} || ($args->{tls} ? 443 : 80),
+        Proto    => $args->{proto},
+        Type     => $args->{proto} eq 'udp' ? SOCK_DGRAM : SOCK_STREAM,
+        %{$args->{args} || {}}
+    );
+
+    # Socket
+    my $class = IPV6 ? 'IO::Socket::IP' : 'IO::Socket::INET';
+    return unless my $socket = $args->{socket} || $class->new(%options);
+    $c->{socket} = $socket;
+    $self->{_reverse}->{$socket} = $id;
+
+    # File descriptor
+    return unless defined(my $fd = fileno $socket);
+    $self->{_fds}->{$fd} = $id;
+
+    # Non-blocking
+    $socket->blocking(0);
+
+    # Disable Nagle's algorithm
+    setsockopt $socket, IPPROTO_TCP, TCP_NODELAY, 1;
+
+    # Timer
+    $c->{connect_timer} =
+      $self->timer($self->connect_timeout =>
+          sub { shift->_error($id, 'Connect timeout.') });
+
+    # Add socket to poll
+    $self->_not_writing($id);
+
+    # Start TLS
+    if ($args->{tls}) { $self->start_tls($id => $args) }
+}
+
 sub _drop_immediately {
     my ($self, $id) = @_;
 
@@ -717,6 +949,7 @@ sub _drop_immediately {
 
     # Delete connection
     my $c = delete $self->{_cs}->{$id};
+    delete $self->{_reverse}->{$id};
 
     # Drop listen socket
     if (!$c && ($c = delete $self->{_listen}->{$id})) {
@@ -861,7 +1094,7 @@ sub _prepare_accept {
     # Remove timeout
     $self->_drop_immediately(delete $c->{accept_timer});
 
-    # Non blocking
+    # Non-blocking
     $c->{socket}->blocking(0);
 
     # Add socket to poll
@@ -893,7 +1126,8 @@ sub _prepare_connect {
     my $c = $self->{_cs}->{$id};
 
     # Not yet connected
-    return unless $c->{socket}->connected;
+    return unless my $socket = $c->{socket};
+    if ($socket->can('connected')) { return unless $socket->connected }
 
     # Connected
     delete $c->{connecting};
@@ -1023,12 +1257,7 @@ sub _read {
     my ($self, $id) = @_;
 
     # Listen socket (new connection)
-    my $found;
-    my $listen = $self->{_listen} || {};
-    if (my $l = $listen->{$id}) { $found = $l->{socket} }
-
-    # Accept new connection
-    return $self->_accept($found) if $found;
+    if (my $l = $self->{_listen}->{$id}) { $self->_accept($l->{socket}) }
 
     # Connection
     my $c = $self->{_cs}->{$id};
@@ -1279,7 +1508,7 @@ __END__
 
 =head1 NAME
 
-Mojo::IOLoop - Minimalistic Reactor For TCP Clients And Servers
+Mojo::IOLoop - Minimalistic Reactor For Non-Blocking TCP Clients And Servers
 
 =head1 SYNOPSIS
 
@@ -1334,8 +1563,8 @@ Mojo::IOLoop - Minimalistic Reactor For TCP Clients And Servers
 =head1 DESCRIPTION
 
 L<Mojo::IOLoop> is a very minimalistic reactor that has been reduced to the
-absolute minimal feature set required to build solid and scalable TCP clients
-and servers.
+absolute minimal feature set required to build solid and scalable non
+blocking TCP clients and servers.
 
 Optional modules L<IO::KQueue>, L<IO::Epoll>, L<IO::Socket::IP> and
 L<IO::Socket::SSL> are supported transparently and used if installed.
@@ -1353,7 +1582,7 @@ L<Mojo::IOLoop> implements the following attributes.
     $loop       = $loop->accept_timeout(5);
 
 Maximum time in seconds a connection can take to be accepted before being
-dropped, defaults to C<5>.
+dropped, defaults to C<3>.
 
 =head2 C<connect_timeout>
 
@@ -1361,7 +1590,24 @@ dropped, defaults to C<5>.
     $loop       = $loop->connect_timeout(5);
 
 Maximum time in seconds a conenction can take to be connected before being
-dropped, defaults to C<5>.
+dropped, defaults to C<3>.
+
+=head2 C<dns_server>
+
+    my $server = $loop->dns_server;
+    $loop      = $loop->dns_server('8.8.8.8');
+
+C<DNS> server to use for non-blocking lookups, defaults to the value of
+C<MOJO_DNS_SERVER>, auto detection or C<8.8.8.8>.
+Note that this attribute is EXPERIMENTAL and might change without warning!
+
+=head2 C<dns_timeout>
+
+    my $timeout = $loop->dns_timeout;
+    $loop       = $loop->dns_timeout(5);
+
+Maximum time in seconds a C<DNS> lookup can take, defaults to C<3>.
+Note that this attribute is EXPERIMENTAL and might change without warning!
 
 =head2 C<max_connections>
 
@@ -1428,7 +1674,7 @@ Note that exceptions in this callback are not captured.
 
 Maximum time in seconds our loop waits for new events to happen, defaults to
 C<0.25>.
-Note that a value of C<0> would make the loop non blocking.
+Note that a value of C<0> would make the loop non-blocking.
 
 =head1 METHODS
 
@@ -1488,6 +1734,10 @@ Callback to be invoked if new data arrives on the connection.
 
 Port to connect to.
 
+=item C<proto>
+
+Protocol to use, defaults to C<tcp>.
+
 =item C<socket>
 
 Use an already prepared socket handle.
@@ -1616,6 +1866,18 @@ The local port.
 
 =back
 
+=head2 C<lookup>
+
+    $loop = $loop->lookup('mojolicio.us' => sub {...});
+
+Lookup C<IPv4> or C<IPv6> address for domain.
+Note that this method is EXPERIMENTAL and might change without warning!
+
+    $loop->lookup('mojolicio.us' => sub {
+        my ($loop, $address) = @_;
+        print "Address: $address\n";
+    });
+
 =head2 C<on_error>
 
     $loop = $loop->on_error($id => sub {...});
@@ -1670,6 +1932,13 @@ The remote port.
 
 =back
 
+=head2 C<resolve>
+
+    $loop = $loop->resolve('mojolicio.us', 'A', sub {...});
+
+Resolve domain into C<A>, C<AAAA> or C<TXT> records.
+Note that this method is EXPERIMENTAL and might change without warning!
+
 =head2 C<singleton>
 
     my $loop = Mojo::IOLoop->singleton;
